@@ -30,7 +30,6 @@
 #include <udisksdaemon.h>
 #include <udisksspawnedjob.h>
 #include <udisksthreadedjob.h>
-#include <udiskspersistentstore.h>
 
 #include "testutil.h"
 
@@ -69,7 +68,7 @@ test_spawned_job_successful (void)
 {
   UDisksSpawnedJob *job;
 
-  job = udisks_spawned_job_new ("/bin/true", NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new ("/bin/true", NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_success), NULL);
   g_object_unref (job);
 }
@@ -81,9 +80,9 @@ test_spawned_job_failure (void)
 {
   UDisksSpawnedJob *job;
 
-  job = udisks_spawned_job_new ("/bin/false", NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new ("/bin/false", NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Command-line `/bin/false' exited with non-zero exit status 1: ");
+                             (gpointer) "Command-line `/bin/false' exited with non-zero exit status 1: ");
   g_object_unref (job);
 }
 
@@ -94,9 +93,9 @@ test_spawned_job_missing_program (void)
 {
   UDisksSpawnedJob *job;
 
-  job = udisks_spawned_job_new ("/path/to/unknown/file", NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new ("/path/to/unknown/file", NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Error spawning command-line `/path/to/unknown/file': Failed to execute child process \"/path/to/unknown/file\" (No such file or directory) (g-exec-error-quark, 8)");
+                             (gpointer) "Error spawning command-line `/path/to/unknown/file': Failed to execute child process \"/path/to/unknown/file\" (No such file or directory) (g-exec-error-quark, 8)");
   g_object_unref (job);
 }
 
@@ -110,9 +109,9 @@ test_spawned_job_cancelled_at_start (void)
 
   cancellable = g_cancellable_new ();
   g_cancellable_cancel (cancellable);
-  job = udisks_spawned_job_new ("/bin/true", NULL, getuid (), geteuid (), cancellable);
+  job = udisks_spawned_job_new ("/bin/true", NULL, getuid (), geteuid (), NULL, cancellable);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Operation was cancelled (g-io-error-quark, 19)");
+                             (gpointer) "Operation was cancelled (g-io-error-quark, 19)");
   g_object_unref (job);
   g_object_unref (cancellable);
 }
@@ -135,11 +134,11 @@ test_spawned_job_cancelled_midway (void)
   GCancellable *cancellable;
 
   cancellable = g_cancellable_new ();
-  job = udisks_spawned_job_new ("/bin/sleep 0.5", NULL, getuid (), geteuid (), cancellable);
+  job = udisks_spawned_job_new ("/bin/sleep 0.5", NULL, getuid (), geteuid (), NULL, cancellable);
   g_timeout_add (10, on_timeout, cancellable); /* 10 msec */
   g_main_loop_run (loop);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Operation was cancelled (g-io-error-quark, 19)");
+                             (gpointer) "Operation was cancelled (g-io-error-quark, 19)");
   g_object_unref (job);
   g_object_unref (cancellable);
 }
@@ -167,11 +166,11 @@ test_spawned_job_override_signal_handler (void)
   UDisksSpawnedJob *job;
   gboolean handler_ran;
 
-  job = udisks_spawned_job_new ("/path/to/unknown/file", NULL, getuid (), geteuid (), NULL /* GCancellable */);
+  job = udisks_spawned_job_new ("/path/to/unknown/file", NULL, getuid (), geteuid (), NULL, NULL /* GCancellable */);
   handler_ran = FALSE;
   g_signal_connect (job, "spawned-job-completed", G_CALLBACK (on_spawned_job_completed), &handler_ran);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Error spawning command-line `/path/to/unknown/file': Failed to execute child process \"/path/to/unknown/file\" (No such file or directory) (g-exec-error-quark, 8)");
+                             (gpointer) "Error spawning command-line `/path/to/unknown/file': Failed to execute child process \"/path/to/unknown/file\" (No such file or directory) (g-exec-error-quark, 8)");
   g_assert (handler_ran);
   g_object_unref (job);
 }
@@ -183,7 +182,7 @@ test_spawned_job_premature_termination (void)
 {
   UDisksSpawnedJob *job;
 
-  job = udisks_spawned_job_new ("/bin/sleep 1000", NULL, getuid (), geteuid (), NULL /* GCancellable */);
+  job = udisks_spawned_job_new ("/bin/sleep 1000", NULL, getuid (), geteuid (), NULL, NULL /* GCancellable */);
   g_object_unref (job);
 }
 
@@ -214,7 +213,7 @@ test_spawned_job_read_stdout (void)
   gchar *s;
 
   s = g_strdup_printf (UDISKS_TEST_DIR "/udisks-test-helper 0");
-  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "spawned-job-completed", G_CALLBACK (read_stdout_on_spawned_job_completed), NULL);
   g_object_unref (job);
   g_free (s);
@@ -247,7 +246,7 @@ test_spawned_job_read_stderr (void)
   gchar *s;
 
   s = g_strdup_printf (UDISKS_TEST_DIR "/udisks-test-helper 1");
-  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "spawned-job-completed", G_CALLBACK (read_stderr_on_spawned_job_completed), NULL);
   g_object_unref (job);
   g_free (s);
@@ -278,14 +277,14 @@ test_spawned_job_exit_status (void)
   gchar *s;
 
   s = g_strdup_printf (UDISKS_TEST_DIR "/udisks-test-helper 2");
-  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "spawned-job-completed", G_CALLBACK (exit_status_on_spawned_job_completed),
                              GINT_TO_POINTER (1));
   g_object_unref (job);
   g_free (s);
 
   s = g_strdup_printf (UDISKS_TEST_DIR "/udisks-test-helper 3");
-  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "spawned-job-completed", G_CALLBACK (exit_status_on_spawned_job_completed),
                              GINT_TO_POINTER (2));
   g_object_unref (job);
@@ -301,17 +300,17 @@ test_spawned_job_abnormal_termination (void)
   gchar *s;
 
   s = g_strdup_printf (UDISKS_TEST_DIR "/udisks-test-helper 4");
-  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Command-line `./udisks-test-helper 4' was signaled with signal SIGSEGV (11): "
+                             (gpointer) "Command-line `./udisks-test-helper 4' was signaled with signal SIGSEGV (11): "
                              "OK, deliberately causing a segfault\n");
   g_object_unref (job);
   g_free (s);
 
   s = g_strdup_printf (UDISKS_TEST_DIR "/udisks-test-helper 5");
-  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Command-line `./udisks-test-helper 5' was signaled with signal SIGABRT (6): "
+                             (gpointer) "Command-line `./udisks-test-helper 5' was signaled with signal SIGABRT (6): "
                              "OK, deliberately abort()'ing\n");
   g_object_unref (job);
   g_free (s);
@@ -350,7 +349,7 @@ test_spawned_job_binary_output (void)
   gchar *s;
 
   s = g_strdup_printf (UDISKS_TEST_DIR "/udisks-test-helper 6");
-  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new (s, NULL, getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "spawned-job-completed", G_CALLBACK (binary_output_on_spawned_job_completed), NULL);
   g_object_unref (job);
   g_free (s);
@@ -381,7 +380,7 @@ test_spawned_job_input_string (void)
   gchar *s;
 
   s = g_strdup_printf (UDISKS_TEST_DIR "/udisks-test-helper 7");
-  job = udisks_spawned_job_new (s, "foobar", getuid (), geteuid (), NULL);
+  job = udisks_spawned_job_new (s, "foobar", getuid (), geteuid (), NULL, NULL);
   _g_assert_signal_received (job, "spawned-job-completed", G_CALLBACK (input_string_on_spawned_job_completed), NULL);
   g_object_unref (job);
   g_free (s);
@@ -404,7 +403,7 @@ test_threaded_job_successful (void)
 {
   UDisksThreadedJob *job;
 
-  job = udisks_threaded_job_new (threaded_job_successful_func, NULL, NULL, NULL);
+  job = udisks_threaded_job_new (threaded_job_successful_func, NULL, NULL, NULL, NULL);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_success), NULL);
   g_object_unref (job);
 }
@@ -430,9 +429,9 @@ test_threaded_job_failure (void)
 {
   UDisksThreadedJob *job;
 
-  job = udisks_threaded_job_new (threaded_job_failure_func, NULL, NULL, NULL);
+  job = udisks_threaded_job_new (threaded_job_failure_func, NULL, NULL, NULL, NULL);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Threaded job failed with error: some error (g-key-file-error-quark, 5)");
+                             (gpointer) "Threaded job failed with error: some error (g-key-file-error-quark, 5)");
   g_object_unref (job);
 }
 
@@ -446,9 +445,9 @@ test_threaded_job_cancelled_at_start (void)
 
   cancellable = g_cancellable_new ();
   g_cancellable_cancel (cancellable);
-  job = udisks_threaded_job_new (threaded_job_successful_func, NULL, NULL, cancellable);
+  job = udisks_threaded_job_new (threaded_job_successful_func, NULL, NULL, NULL, cancellable);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Threaded job failed with error: Operation was cancelled (g-io-error-quark, 19)");
+                             (gpointer) "Threaded job failed with error: Operation was cancelled (g-io-error-quark, 19)");
   g_object_unref (job);
   g_object_unref (cancellable);
 }
@@ -485,11 +484,11 @@ test_threaded_job_cancelled_midway (void)
 
   cancellable = g_cancellable_new ();
   count = 0;
-  job = udisks_threaded_job_new (threaded_job_sleep_until_cancelled, &count, NULL, cancellable);
+  job = udisks_threaded_job_new (threaded_job_sleep_until_cancelled, &count, NULL, NULL, cancellable);
   g_timeout_add (10, on_timeout, cancellable); /* 10 msec */
   g_main_loop_run (loop);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Threaded job failed with error: Operation was cancelled (g-io-error-quark, 19)");
+                             (gpointer) "Threaded job failed with error: Operation was cancelled (g-io-error-quark, 19)");
   g_assert_cmpint (count, >, 0);
   g_object_unref (job);
   g_object_unref (cancellable);
@@ -518,177 +517,23 @@ test_threaded_job_override_signal_handler (void)
   UDisksThreadedJob *job;
   gboolean handler_ran;
 
-  job = udisks_threaded_job_new (threaded_job_failure_func, NULL, NULL, NULL);
+  job = udisks_threaded_job_new (threaded_job_failure_func, NULL, NULL, NULL, NULL);
   handler_ran = FALSE;
   g_signal_connect (job, "threaded-job-completed", G_CALLBACK (on_threaded_job_completed), &handler_ran);
   _g_assert_signal_received (job, "completed", G_CALLBACK (on_completed_expect_failure),
-                             "Threaded job failed with error: some error (g-key-file-error-quark, 5)");
+                             (gpointer) "Threaded job failed with error: some error (g-key-file-error-quark, 5)");
   g_assert (handler_ran);
   g_object_unref (job);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-static void
-check_store (UDisksPersistentStore *store)
-{
-  GError *error;
-  GVariant *value;
-  gchar *s;
-
-  error = NULL;
-  value = udisks_persistent_store_get (store,
-                                       UDISKS_PERSISTENT_FLAGS_NORMAL_STORE,
-                                       "non-existing",
-                                       G_VARIANT_TYPE ("(sss)"),
-                                       &error);
-  g_assert_no_error (error);
-  g_assert (value == NULL);
-
-  error = NULL;
-  value = udisks_persistent_store_get (store,
-                                       UDISKS_PERSISTENT_FLAGS_TEMPORARY_STORE,
-                                       "non-existing",
-                                       G_VARIANT_TYPE ("(sss)"),
-                                       &error);
-  g_assert_no_error (error);
-  g_assert (value == NULL);
-
-  error = NULL;
-  value = udisks_persistent_store_get (store,
-                                       UDISKS_PERSISTENT_FLAGS_NORMAL_STORE,
-                                       "ducks",
-                                       G_VARIANT_TYPE ("(sss)"),
-                                       &error);
-  g_assert_no_error (error);
-  g_assert (value != NULL);
-  s = g_variant_print (value, TRUE);
-  g_assert_cmpstr (s, ==, "('Huey', 'Dewey', 'Louie')");
-  g_free (s);
-  g_variant_unref (value);
-
-  error = NULL;
-  value = udisks_persistent_store_get (store,
-                                       UDISKS_PERSISTENT_FLAGS_TEMPORARY_STORE,
-                                       "ducks",
-                                       G_VARIANT_TYPE ("(sss)"),
-                                       &error);
-  g_assert_no_error (error);
-  g_assert (value != NULL);
-  s = g_variant_print (value, TRUE);
-  g_assert_cmpstr (s, ==, "('Rip', 'Rap', 'Rup')");
-  g_free (s);
-  g_variant_unref (value);
-}
-
-static void
-cleanup_dir (const gchar *dirname)
-{
-  gchar *command;
-
-  g_assert (g_str_has_prefix (dirname, "/tmp/test-udisks-persistent-store-"));
-  g_assert (strstr (dirname, "\"") == NULL);
-  command = g_strdup_printf ("rm -rf \"%s\"", dirname);
-  g_assert_cmpint (system (command), ==, 0);
-  g_free (command);
-}
-
-static void
-test_persistent_store (void)
-{
-  UDisksPersistentStore *store;
-  gchar *dirname;
-  gchar *dirname_temp;
-  GError *error;
-  gboolean rc;
-  GVariant *value;
-  gchar *s;
-
-  dirname = g_strdup ("/tmp/test-udisks-persistent-store-XXXXXX");
-  if (mkdtemp (dirname) == 0)
-    {
-      g_warning ("Error creating temp dir with template %s: %m", dirname);
-      g_assert_not_reached ();
-    }
-
-  dirname_temp = g_strdup ("/tmp/test-udisks-persistent-store-temp-XXXXXX");
-  if (mkdtemp (dirname_temp) == 0)
-    {
-      g_warning ("Error creating temp dir with template %s: %m", dirname_temp);
-      g_assert_not_reached ();
-    }
-
-  store = udisks_persistent_store_new (dirname, dirname_temp);
-
-  error = NULL;
-  rc = udisks_persistent_store_set (store,
-                                    UDISKS_PERSISTENT_FLAGS_NORMAL_STORE,
-                                    "ducks",
-                                    G_VARIANT_TYPE ("(sss)"),
-                                    g_variant_new ("(sss)", "Huey", "Dewey", "Louie"),
-                                    &error);
-  g_assert_no_error (error);
-  g_assert (rc);
-
-  error = NULL;
-  rc = udisks_persistent_store_set (store,
-                                    UDISKS_PERSISTENT_FLAGS_TEMPORARY_STORE,
-                                    "ducks",
-                                    G_VARIANT_TYPE ("(sss)"),
-                                    g_variant_new ("(sss)", "Rip", "Rap", "Rup"),
-                                    &error);
-  g_assert_no_error (error);
-  g_assert (rc);
-
-  check_store (store);
-
-  /* now nuke the store object and create a new object - we should see
-   * the same data (since it's persistent!)
-   */
-  g_object_unref (store);
-  store = udisks_persistent_store_new (dirname, dirname_temp);
-  check_store (store);
-
-  /* check we can replace one of the entries */
-  error = NULL;
-  rc = udisks_persistent_store_set (store,
-                                    UDISKS_PERSISTENT_FLAGS_NORMAL_STORE,
-                                    "ducks",
-                                    G_VARIANT_TYPE ("(sss)"),
-                                    g_variant_new ("(sss)", "April", "May", "June"),
-                                    &error);
-  g_assert_no_error (error);
-  g_assert (rc);
-  /* and we can read it back */
-  error = NULL;
-  value = udisks_persistent_store_get (store,
-                                       UDISKS_PERSISTENT_FLAGS_NORMAL_STORE,
-                                       "ducks",
-                                       G_VARIANT_TYPE ("(sss)"),
-                                       &error);
-  g_assert_no_error (error);
-  g_assert (value != NULL);
-  s = g_variant_print (value, TRUE);
-  g_assert_cmpstr (s, ==, "('April', 'May', 'June')");
-  g_free (s);
-  g_variant_unref (value);
-
-  /* ok, done */
-  g_object_unref (store);
-
-  cleanup_dir (dirname);
-  cleanup_dir (dirname_temp);
-}
-
-/* ---------------------------------------------------------------------------------------------------- */
-
 int
 main (int    argc,
       char **argv)
 {
   int ret;
 
-  g_type_init ();
   g_test_init (&argc, &argv, NULL);
 
   loop = g_main_loop_new (NULL, FALSE);
@@ -712,7 +557,6 @@ main (int    argc,
   g_test_add_func ("/udisks/daemon/threaded_job/cancelled_at_start", test_threaded_job_cancelled_at_start);
   g_test_add_func ("/udisks/daemon/threaded_job/cancelled_midway", test_threaded_job_cancelled_midway);
   g_test_add_func ("/udisks/daemon/threaded_job/override_signal_handler", test_threaded_job_override_signal_handler);
-  g_test_add_func ("/udisks/daemon/persistent_store", test_persistent_store);
 
   ret = g_test_run();
 
