@@ -44,6 +44,8 @@
 #if defined(HAVE_LIBSYSTEMD_LOGIN)
 #include <systemd/sd-daemon.h>
 #include <systemd/sd-login.h>
+
+#define LOGIND_AVAILABLE() (access("/run/systemd/seats/", F_OK) >= 0)
 #endif
 
 /**
@@ -155,7 +157,7 @@ udisks_safe_append_to_object_path (GString      *str,
       else
         {
           /* Escape bytes not in [A-Z][a-z][0-9] as _<hex-with-two-digits> */
-          g_string_append_printf (str, "_%02x", c);
+          g_string_append_printf (str, "_%02x", (guint) c);
         }
     }
 }
@@ -635,7 +637,31 @@ udisks_daemon_util_check_authorization_sync (UDisksDaemon          *daemon,
       _safe_polkit_details_insert (details, "drive.model", udisks_drive_get_model (drive));
       _safe_polkit_details_insert (details, "drive.revision", udisks_drive_get_revision (drive));
       if (udisks_drive_get_removable (drive))
-        polkit_details_insert (details, "drive.removable", "true");
+        {
+          const gchar *const *media_compat;
+          GString *media_compat_str;
+          const gchar *sep = ",";
+
+          polkit_details_insert (details, "drive.removable", "true");
+          _safe_polkit_details_insert (details, "drive.removable.bus", udisks_drive_get_connection_bus (drive));
+
+          media_compat_str = g_string_new (NULL);
+          media_compat = udisks_drive_get_media_compatibility (drive);
+          if (media_compat)
+            {
+              guint i;
+
+              for (i = 0; media_compat[i] && strlen(media_compat[i]); i++)
+                {
+                  if (i)
+                    g_string_append (media_compat_str, sep);
+                  g_string_append (media_compat_str, media_compat[i]);
+                }
+            }
+
+          _safe_polkit_details_insert (details, "drive.removable.media", media_compat_str->str);
+          g_string_free (media_compat_str, TRUE);
+        }
     }
 
   if (block != NULL)
@@ -804,7 +830,7 @@ udisks_daemon_util_get_caller_uid_sync (UDisksDaemon            *daemon,
     {
       struct passwd pwstruct;
       gchar pwbuf[8192];
-      static struct passwd *pw;
+      struct passwd *pw = NULL;
       int rc;
 
       rc = getpwuid_r (uid, &pwstruct, pwbuf, sizeof pwbuf, &pw);
@@ -814,6 +840,7 @@ udisks_daemon_util_get_caller_uid_sync (UDisksDaemon            *daemon,
                        UDISKS_ERROR,
                        UDISKS_ERROR_FAILED,
                        "User with uid %d does not exist", (gint) uid);
+          goto out;
         }
       else if (pw == NULL)
         {
@@ -1039,8 +1066,8 @@ udisks_daemon_util_on_same_seat (UDisksDaemon          *daemon,
   UDisksObject *drive_object = NULL;
   UDisksDrive *drive = NULL;
 
-  /* if we haven't booted with systemd, assume it's always the same seat */
-  if (sd_booted () <= 0)
+  /* if we don't have logind, assume it's always the same seat */
+  if (!LOGIND_AVAILABLE())
     return TRUE;
 
   if (UDISKS_IS_LINUX_BLOCK_OBJECT (object))
@@ -1118,7 +1145,7 @@ udisks_daemon_util_hexdump (gconstpointer data, gsize len)
           if (m > n && (m%4) == 0)
             g_string_append_c (ret, ' ');
           if (m < len)
-            g_string_append_printf (ret, "%02x ", bdata[m]);
+            g_string_append_printf (ret, "%02x ", (guint) bdata[m]);
           else
             g_string_append (ret, "   ");
         }
