@@ -88,6 +88,8 @@ struct _UDisksLinuxDriveAta
   UDisksThreadedJob *selftest_job;
 
   gboolean     secure_erase_in_progress;
+  unsigned long drive_read, drive_write;
+  gboolean     standby_enabled;
 };
 
 struct _UDisksLinuxDriveAtaClass
@@ -427,6 +429,68 @@ selftest_status_to_string (SkSmartSelfTestExecutionStatus status)
   return ret;
 }
 
+static gboolean get_pm_state (UDisksLinuxDevice *device, GError **error, guchar *count)
+{
+  int fd;
+  gboolean rc = FALSE;
+  /* ATA8: 7.8 CHECK POWER MODE - E5h, Non-Data */
+  UDisksAtaCommandInput input = {.command = 0xe5};
+  UDisksAtaCommandOutput output = {0};
+
+  fd = open (g_udev_device_get_device_file (device->udev_device), O_RDONLY|O_NONBLOCK);
+  if (fd == -1)
+    {
+      g_set_error (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                   "Error opening device file %s: %m",
+                   g_udev_device_get_device_file (device->udev_device));
+      goto out;
+    }
+
+  if (!udisks_ata_send_command_sync (fd,
+                                     -1,
+                                     UDISKS_ATA_COMMAND_PROTOCOL_NONE,
+                                     &input,
+                                     &output,
+                                     error))
+    {
+      g_prefix_error (error, "Error sending ATA command CHECK POWER MODE: ");
+      goto out;
+    }
+  /* count field is used for the state, see ATA8: table 102 */
+  *count = output.count;
+  rc = TRUE;
+ out:
+  if (fd != -1)
+    close (fd);
+  return rc;
+}
+
+static gboolean update_io_stats (UDisksLinuxDriveAta *drive, UDisksLinuxDevice *device)
+{
+  const gchar *drivepath = g_udev_device_get_sysfs_path (device->udev_device);
+  gchar statpath[PATH_MAX];
+  unsigned long drive_read, drive_write;
+  FILE *statf;
+  gboolean noio = FALSE;
+  snprintf (statpath, sizeof(statpath), "%s/stat", drivepath);
+  statf = fopen (statpath, "r");
+  if (statf == NULL)
+    {
+      udisks_warning ("Failed to open %s\n", statpath);
+    }
+  else
+    {
+      fscanf (statf, "%lu %*u %*u %*u %lu", &drive_read, &drive_write);
+      fclose (statf);
+      noio = drive_read == drive->drive_read && drive_write == drive->drive_write;
+      udisks_debug ("drive_read=%lu, drive_write=%lu, old_drive_read=%lu, old_drive_write=%lu\n",
+                    drive_read, drive_write, drive->drive_read, drive->drive_write);
+      drive->drive_read = drive_read;
+      drive->drive_write = drive_write;
+    }
+  return noio;
+}
+
 /**
  * udisks_linux_drive_ata_refresh_smart_sync:
  * @drive: The #UDisksLinuxDriveAta to refresh.
@@ -519,26 +583,15 @@ udisks_linux_drive_ata_refresh_smart_sync (UDisksLinuxDriveAta  *drive,
     }
   else
     {
-      if (sk_disk_open (g_udev_device_get_device_file (device->udev_device), &d) != 0)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "sk_disk_open: %m");
-          goto out;
-        }
-
-      if (sk_disk_check_sleep_mode (d, &awake) != 0)
-        {
-          g_set_error (error,
-                       UDISKS_ERROR,
-                       UDISKS_ERROR_FAILED,
-                       "sk_disk_check_sleep_mode: %m");
-          goto out;
-        }
-
+      guchar count;
+      gboolean noio = FALSE;
+      if (!get_pm_state(device, error, &count))
+        goto out;
+      awake = count == 0xFF || count == 0x80;
+      if (drive->standby_enabled)
+        noio = update_io_stats (drive, device);
       /* don't wake up disk unless specically asked to */
-      if (nowakeup && !awake)
+      if (nowakeup && (!awake || noio))
         {
           g_set_error (error,
                        UDISKS_ERROR,
@@ -548,6 +601,15 @@ udisks_linux_drive_ata_refresh_smart_sync (UDisksLinuxDriveAta  *drive,
         }
     }
 
+  if (sk_disk_open (g_udev_device_get_device_file (device->udev_device), &d) != 0)
+    {
+      g_set_error (error,
+                   UDISKS_ERROR,
+                   UDISKS_ERROR_FAILED,
+                   "sk_disk_open: %m");
+      goto out;
+    }
+
   if (sk_disk_smart_read_data (d) != 0)
     {
       g_set_error (error,
@@ -603,6 +665,8 @@ udisks_linux_drive_ata_refresh_smart_sync (UDisksLinuxDriveAta  *drive,
   update_smart (drive, device);
 
   ret = TRUE;
+  /* update stats again to account for the IO we just did to read the SMART info */
+  update_io_stats (drive, device);
 
  out:
   g_clear_object (&device);
@@ -637,7 +701,7 @@ udisks_linux_drive_ata_smart_selftest_sync (UDisksLinuxDriveAta     *drive,
                                             GError                 **error)
 {
   UDisksLinuxDriveObject  *object;
-  UDisksLinuxDevice *device;
+  UDisksLinuxDevice *device = NULL;
   SkDisk *d = NULL;
   gboolean ret = FALSE;
   SkSmartSelfTest test;
@@ -1184,10 +1248,10 @@ handle_pm_get_state (UDisksDriveAta        *_drive,
   UDisksLinuxDriveObject  *object = NULL;
   UDisksDaemon *daemon;
   UDisksLinuxDevice *device = NULL;
-  gint fd = -1;
   GError *error = NULL;
   const gchar *message;
   const gchar *action_id;
+  guchar count;
 
   object = udisks_daemon_util_dup_object (drive, &error);
   if (object == NULL)
@@ -1244,38 +1308,12 @@ handle_pm_get_state (UDisksDriveAta        *_drive,
                                              "No udev device");
       goto out;
     }
-
-  fd = open (g_udev_device_get_device_file (device->udev_device), O_RDONLY|O_NONBLOCK);
-  if (fd == -1)
-    {
-      g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                                             "Error opening device file %s: %m",
-                                             g_udev_device_get_device_file (device->udev_device));
-      goto out;
-    }
-
-  {
-    /* ATA8: 7.8 CHECK POWER MODE - E5h, Non-Data */
-    UDisksAtaCommandInput input = {.command = 0xe5};
-    UDisksAtaCommandOutput output = {0};
-    if (!udisks_ata_send_command_sync (fd,
-                                       -1,
-                                       UDISKS_ATA_COMMAND_PROTOCOL_NONE,
-                                       &input,
-                                       &output,
-                                       &error))
-      {
-        g_prefix_error (&error, "Error sending ATA command CHECK POWER MODE: ");
-        g_dbus_method_invocation_take_error (invocation, error);
-        goto out;
-      }
-    /* count field is used for the state, see ATA8: table 102 */
-    udisks_drive_ata_complete_pm_get_state (_drive, invocation, output.count);
-  }
+  if (get_pm_state (device, &error, &count))
+    udisks_drive_ata_complete_pm_get_state (_drive, invocation, count);
+  else
+    g_dbus_method_invocation_take_error (invocation, error);
 
  out:
-  if (fd != -1)
-    close (fd);
   g_clear_object (&device);
   g_clear_object (&object);
   return TRUE; /* returning TRUE means that we handled the method invocation */
@@ -1655,6 +1693,7 @@ apply_configuration_thread_func (gpointer user_data)
           udisks_notice ("Set standby timer to %s (value %d) on %s [%s]",
                          pretty, data->ata_pm_standby, device_file, udisks_drive_get_id (data->drive));
           g_free (pretty);
+          data->ata->standby_enabled = data->ata_pm_standby != 0;
         }
     }
 
@@ -1678,7 +1717,7 @@ apply_configuration_thread_func (gpointer user_data)
                                          &error))
         {
           udisks_error ("Error sending ATA command SET FEATURES, sub-command 0x%02x (ata_apm_level=%d) to %s: %s (%s, %d)",
-                        input.feature, data->ata_apm_level, device_file,
+                        (guint) input.feature, data->ata_apm_level, device_file,
                         error->message, g_quark_to_string (error->domain), error->code);
           g_clear_error (&error);
         }
@@ -1709,7 +1748,7 @@ apply_configuration_thread_func (gpointer user_data)
                                          &error))
         {
           udisks_error ("Error sending ATA command SET FEATURES, sub-command 0x%02x (ata_aam_level=%d) to %s: %s (%s, %d)",
-                        input.feature, data->ata_aam_level, device_file,
+                        (guint) input.feature, data->ata_aam_level, device_file,
                         error->message, g_quark_to_string (error->domain), error->code);
           g_clear_error (&error);
         }
@@ -1737,7 +1776,7 @@ apply_configuration_thread_func (gpointer user_data)
                                          &error))
         {
           udisks_error ("Error sending ATA command SET FEATURES, sub-command 0x%02x to %s: %s (%s, %d)",
-                        input.feature, device_file,
+                        (guint) input.feature, device_file,
                         error->message, g_quark_to_string (error->domain), error->code);
           g_clear_error (&error);
         }
@@ -1943,7 +1982,7 @@ udisks_linux_drive_ata_secure_erase_sync (UDisksLinuxDriveAta     *drive,
   /* First get the IDENTIFY data directly from the drive, for sanity checks */
   {
     /* ATA8: 7.16 IDENTIFY DEVICE - ECh, PIO Data-In */
-    UDisksAtaCommandInput input = {.command = 0xec};
+    UDisksAtaCommandInput input = {.command = 0xec, .count = 1};
     UDisksAtaCommandOutput output = {.buffer = identify.buf, .buffer_size = sizeof (identify.buf)};
     if (!udisks_ata_send_command_sync (fd,
                                        -1,
