@@ -538,7 +538,6 @@ handle_mdraid_create (UDisksManager         *_object,
 {
   UDisksLinuxManager *manager = UDISKS_LINUX_MANAGER (_object);
   UDisksObject *array_object = NULL;
-  WaitForArrayData wait_data;
   uid_t caller_uid;
   GError *error = NULL;
   const gchar *message;
@@ -643,7 +642,7 @@ handle_mdraid_create (UDisksManager         *_object,
           g_dbus_method_invocation_return_error (invocation,
                                                  UDISKS_ERROR,
                                                  UDISKS_ERROR_FAILED,
-                                                 "Invalid object path %s at index %d",
+                                                 "Invalid object path %s at index %u",
                                                  arg_blocks[n], n);
           goto out;
         }
@@ -654,7 +653,7 @@ handle_mdraid_create (UDisksManager         *_object,
           g_dbus_method_invocation_return_error (invocation,
                                                  UDISKS_ERROR,
                                                  UDISKS_ERROR_FAILED,
-                                                 "Object path %s for index %d is not a block device",
+                                                 "Object path %s for index %u is not a block device",
                                                  arg_blocks[n], n);
           goto out;
         }
@@ -737,7 +736,7 @@ handle_mdraid_create (UDisksManager         *_object,
   g_string_append_printf (str, " --level %s", arg_level);
   if (strlen (arg_name) > 0)
     g_string_append_printf (str, " --name \"%s\"", escaped_name);
-  g_string_append_printf (str, " --raid-devices %d", num_devices);
+  g_string_append_printf (str, " --raid-devices %u", num_devices);
   for (l = blocks; l != NULL; l = l->next)
     {
       UDisksBlock *block = UDISKS_BLOCK (l->data);
@@ -778,8 +777,8 @@ handle_mdraid_create (UDisksManager         *_object,
   if (array_object == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for array object after creating /dev/md%d",
-                      wait_data.md_num);
+                      "Error waiting for array object after creating %s",
+                      raid_device_file);
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
     }
