@@ -262,7 +262,7 @@ udisks_client_get_object_info_for_block (UDisksClient     *client,
        *              The %d is the partition number.
        *              The %s is the description for the block device (e.g. "5 GB Block Device").
        */
-      s = g_strdup_printf (C_("part-block", "Partition %d of %s"),
+      s = g_strdup_printf (C_("part-block", "Partition %u of %s"),
                            udisks_partition_get_number (partition), info->description);
       g_free (info->description);
       info->description = s;
@@ -276,7 +276,7 @@ udisks_client_get_object_info_for_block (UDisksClient     *client,
                                      info->description,
                                      udisks_block_get_preferred_device (block));
 
-  info->sort_key = g_strdup_printf ("02_block_%s_%d",
+  info->sort_key = g_strdup_printf ("02_block_%s_%u",
                                     last_segment (g_dbus_object_get_object_path (G_DBUS_OBJECT (info->object))),
                                     partition != NULL ? udisks_partition_get_number (partition) : 0);
 }
@@ -316,7 +316,7 @@ udisks_client_get_object_info_for_loop (UDisksClient     *client,
        *              The %d is the partition number.
        *              The %s is the description for the loop device (e.g. "5 GB Loop Device").
        */
-      s = g_strdup_printf (C_("part-loop", "Partition %d of %s"),
+      s = g_strdup_printf (C_("part-loop", "Partition %u of %s"),
                            udisks_partition_get_number (partition), info->description);
       g_free (info->description);
       info->description = s;
@@ -332,7 +332,7 @@ udisks_client_get_object_info_for_loop (UDisksClient     *client,
                                      info->name,
                                      udisks_block_get_preferred_device (block));
 
-  info->sort_key = g_strdup_printf ("03_loop_%s_%d",
+  info->sort_key = g_strdup_printf ("03_loop_%s_%u",
                                     last_segment (g_dbus_object_get_object_path (G_DBUS_OBJECT (info->object))),
                                     partition != NULL ? udisks_partition_get_number (partition) : 0);
 }
@@ -412,7 +412,7 @@ udisks_client_get_object_info_for_mdraid (UDisksClient     *client,
        *              The %d is the partition number.
        *              The %s is the description for the drive (e.g. "2 TB RAID-5").
        */
-      s = g_strdup_printf (C_("part-raid", "Partition %d of %s"),
+      s = g_strdup_printf (C_("part-raid", "Partition %u of %s"),
                            udisks_partition_get_number (partition), info->description);
       g_free (info->description);
       info->description = s;
@@ -469,7 +469,7 @@ udisks_client_get_object_info_for_mdraid (UDisksClient     *client,
 
   g_clear_object (&block);
 
-  info->sort_key = g_strdup_printf ("01_mdraid_%s_%d", udisks_mdraid_get_uuid (mdraid),
+  info->sort_key = g_strdup_printf ("01_mdraid_%s_%u", udisks_mdraid_get_uuid (mdraid),
                                     partition != NULL ? udisks_partition_get_number (partition) : 0);
 }
 
@@ -511,7 +511,7 @@ udisks_client_get_object_info_for_drive (UDisksClient     *client,
   const gchar *const *media_compat;
   gboolean media_available;
   gboolean media_removable;
-  gboolean rotation_rate;
+  gint rotation_rate;
   guint64 size;
   gchar *size_str;
   guint n;
@@ -691,9 +691,16 @@ udisks_client_get_object_info_for_drive (UDisksClient     *client,
   if (info->icon == NULL)
     {
       if (media_removable)
-        s = g_strdup_printf ("drive-removable-media%s", hyphenated_connection_bus);
+        {
+          s = g_strdup_printf ("drive-removable-media%s", hyphenated_connection_bus);
+        }
       else
-        s = g_strdup_printf ("drive-harddisk%s", hyphenated_connection_bus);
+        {
+          if (rotation_rate == 0)
+            s = g_strdup_printf ("drive-harddisk-solidstate%s", hyphenated_connection_bus);
+          else
+            s = g_strdup_printf ("drive-harddisk%s", hyphenated_connection_bus);
+        }
       info->icon = g_themed_icon_new_with_default_fallbacks (s);
       g_free (s);
     }
@@ -701,29 +708,50 @@ udisks_client_get_object_info_for_drive (UDisksClient     *client,
   if (info->icon_symbolic == NULL)
     {
       if (media_removable)
-        s = g_strdup_printf ("drive-removable-media%s-symbolic", hyphenated_connection_bus);
+        {
+          s = g_strdup_printf ("drive-removable-media%s-symbolic", hyphenated_connection_bus);
+        }
       else
-        s = g_strdup_printf ("drive-harddisk%s-symbolic", hyphenated_connection_bus);
+        {
+          if (rotation_rate == 0)
+            s = g_strdup_printf ("drive-harddisk-solidstate%s-symbolic", hyphenated_connection_bus);
+          else
+            s = g_strdup_printf ("drive-harddisk%s-symbolic", hyphenated_connection_bus);
+        }
       info->icon_symbolic = g_themed_icon_new_with_default_fallbacks (s);
       g_free (s);
     }
   /* fallback for media_icon */
-  if (media_removable && media_available && info->media_icon == NULL)
+  if (media_available && info->media_icon == NULL)
     {
       if (media_removable)
-        s = g_strdup_printf ("drive-removable-media%s", hyphenated_connection_bus);
+        {
+          s = g_strdup_printf ("drive-removable-media%s", hyphenated_connection_bus);
+        }
       else
-        s = g_strdup_printf ("drive-harddisk%s", hyphenated_connection_bus);
+        {
+          if (rotation_rate == 0)
+            s = g_strdup_printf ("drive-harddisk-solidstate%s", hyphenated_connection_bus);
+          else
+            s = g_strdup_printf ("drive-harddisk%s", hyphenated_connection_bus);
+        }
       info->media_icon = g_themed_icon_new_with_default_fallbacks (s);
       g_free (s);
     }
   /* fallback for media_icon_symbolic */
-  if (media_removable && media_available && info->media_icon_symbolic == NULL)
+  if (media_available && info->media_icon_symbolic == NULL)
     {
       if (media_removable)
-        s = g_strdup_printf ("drive-removable-media%s-symbolic", hyphenated_connection_bus);
+        {
+          s = g_strdup_printf ("drive-removable-media%s-symbolic", hyphenated_connection_bus);
+        }
       else
-        s = g_strdup_printf ("drive-harddisk%s-symbolic", hyphenated_connection_bus);
+        {
+          if (rotation_rate == 0)
+            s = g_strdup_printf ("drive-harddisk-solidstate%s-symbolic", hyphenated_connection_bus);
+          else
+            s = g_strdup_printf ("drive-harddisk%s-symbolic", hyphenated_connection_bus);
+        }
       info->media_icon_symbolic = g_themed_icon_new_with_default_fallbacks (s);
       g_free (s);
     }
@@ -801,7 +829,7 @@ udisks_client_get_object_info_for_drive (UDisksClient     *client,
        *              The %d is the partition number.
        *              The %s is the description for the drive (e.g. "2 GB Thumb Drive").
        */
-      s = g_strdup_printf (C_("part-drive", "Partition %d of %s"),
+      s = g_strdup_printf (C_("part-drive", "Partition %u of %s"),
                            udisks_partition_get_number (partition), info->description);
       g_free (info->description);
       info->description = s;
